
package uk.co.badgersinfoil.jactionscript;


/**
 * Definitions of opcodes that have no immidiate data,
 */
public interface ActionConstants {
	public static final Action ADD = new Action("add") {
		public void accept(Visitor v) throws JASVisitorException { v.visitADD(this); }
	};
	public static final Action AND = new Action("and") {
		public void accept(Visitor v) throws JASVisitorException { v.visitAND(this); }
	};
	public static final Action ASCII_TO_CHAR = new Action("asciiToChar") {
		public void accept(Visitor v) throws JASVisitorException { v.visitASCII_TO_CHAR(this); }
	};
	public static final Action ASCII_TO_CHAR_MB = new Action("asciiToCharMB") {
		public void accept(Visitor v) throws JASVisitorException { v.visitASCII_TO_CHAR_MB(this); }
	};
	public static final Action BIT_AND = new Action("bitAnd") {
		public void accept(Visitor v) throws JASVisitorException { v.visitBIT_AND(this); }
	};
	public static final Action BIT_OR = new Action("bitOr") {
		public void accept(Visitor v) throws JASVisitorException { v.visitBIT_OR(this); }
	};
	public static final Action BIT_XOR = new Action("bitXor") {
		public void accept(Visitor v) throws JASVisitorException { v.visitBIT_XOR(this); }
	};
	public static final Action CALL = new Action("call") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCALL(this); }
	};
	public static final Action CALL_FUNCTION = new Action("callFunction") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCALL_FUNCTION(this); }
	};
	public static final Action CALL_METHOD = new Action("callMethod") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCALL_METHOD(this); }
	};
	public static final Action CHAR_MB_TO_ASCII = new Action("charMBToAscii") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCHAR_MB_TO_ASCII(this); }
	};
	public static final Action CHAR_TO_ASCII = new Action("charToAscii") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCHAR_TO_ASCII(this); }
	};
	public static final Action CLONE_SPRITE = new Action("cloneSprite") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCLONE_SPRITE(this); }
	};
	public static final Action CONCAT = new Action("concat") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCONCAT(this); }
	};
	public static final Action CONVERT_TO_NUMBER = new Action("convertToNumber") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCONVERT_TO_NUMBER(this);}
	};
	public static final Action CONVERT_TO_STRING = new Action("convertToString") {
		public void accept(Visitor v) throws JASVisitorException { v.visitCONVERT_TO_STRING(this);}
	};
	public static final Action DECREMENT = new Action("decrement") {
		public void accept(Visitor v) throws JASVisitorException { v.visitDECREMENT(this); }
	};
	public static final Action DEFINE_LOCAL = new Action("defineLocal") {
		public void accept(Visitor v) throws JASVisitorException { v.visitDEFINE_LOCAL(this); }
	};
	public static final Action DEFINE_LOCAL_VALUE = new Action("defineLocalValue") {
		public void accept(Visitor v) throws JASVisitorException {v.visitDEFINE_LOCAL_VALUE(this);}
	};
	public static final Action DELETE_PROPERTY = new Action("deleteProperty") {
		public void accept(Visitor v) throws JASVisitorException { v.visitDELETE_PROPERTY(this); }
	};
	public static final Action DELETE_THREAD_VARS = new Action("deleteThreadVars") {
		public void accept(Visitor v) throws JASVisitorException {v.visitDELETE_THREAD_VARS(this);}
	};
	public static final Action DIVIDE = new Action("divide") {
		public void accept(Visitor v) throws JASVisitorException { v.visitDIVIDE(this); }
	};
	public static final Action DUPLICATE = new Action("duplicate") {
		public void accept(Visitor v) throws JASVisitorException { v.visitDUPLICATE(this); }
	};
	public static final Action END_DRAG = new Action("endDrag") {
		public void accept(Visitor v) throws JASVisitorException { v.visitEND_DRAG(this); }
	};
	public static final Action ENUMERATE = new Action("enumerate") {
		public void accept(Visitor v) throws JASVisitorException { v.visitENUMERATE(this); }
	};
	public static final Action ENUMERATE_OBJECT = new Action("enumerateObject") {
		public void accept(Visitor v) throws JASVisitorException { v.visitENUMERATE_OBJECT(this); }
	};
	public static final Action EQUALS = new Action("equals") {
		public void accept(Visitor v) throws JASVisitorException { v.visitEQUALS(this); }
	};
	public static final Action GET_MEMBER = new Action("getMember") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGET_MEMBER(this); }
	};
	public static final Action GET_PROPERTY = new Action("getProperty") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGET_PROPERTY(this); }
	};
	public static final Action GET_TARGET_PATH = new Action("getTargetPath") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGET_TARGET_PATH(this); }
	};
	public static final Action GET_TIME = new Action("getTime") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGET_TIME(this); }
	};
	// ... see class GetURLAction ...
	// ... see class GetURL2Action ...
	public static final Action GET_VARIABLE = new Action("getVariable") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGET_VARIABLE(this); }
	};
	// ... see class GotoFrame2Action ...
	// ... see class GotoFrameAction ...
	// ... see class GotoLabelAction ...
	public static final Action GREATER_THAN = new Action("greaterThan") {
		public void accept(Visitor v) throws JASVisitorException { v.visitGREATER_THAN(this); }
	};
	// ... see class IfJumpAction ...
	public static final Action INCREMENT = new Action("increment") {
		public void accept(Visitor v) throws JASVisitorException { v.visitINCREMENT(this); }
	};
	public static final Action INIT_ARRAY = new Action("initArray") {
		public void accept(Visitor v) throws JASVisitorException { v.visitINIT_ARRAY(this); }
	};
	public static final Action INIT_OBJECT = new Action("initObject") {
		public void accept(Visitor v) throws JASVisitorException { v.visitINIT_OBJECT(this); }
	};
	public static final Action INSTANCE_OF = new Action("instanceOf") {
		public void accept(Visitor v) throws JASVisitorException { v.visitINSTANCE_OF(this); }
	};
	// ... see class JumpAction ...
	public static final Action LESS_THAN = new Action("lessThan") {
		public void accept(Visitor v) throws JASVisitorException { v.visitLESS_THAN(this); }
	};
	// ... see class LookupAction ...
	// ... see class LookupTableAction ...
	public static final Action MODULO = new Action("modulo") {
		public void accept(Visitor v) throws JASVisitorException { v.visitMODULO(this); }
	};
	public static final Action MULTIPLY = new Action("multiply") {
		public void accept(Visitor v) throws JASVisitorException { v.visitMULTIPLY(this); }
	};
	public static final Action NEW_METHOD = new Action("newMethod") {
		public void accept(Visitor v) throws JASVisitorException { v.visitNEW_METHOD(this); }
	};
	public static final Action NEW_OBJECT = new Action("newObject") {
		public void accept(Visitor v) throws JASVisitorException { v.visitNEW_OBJECT(this); }
	};
	public static final Action NEXT_FRAME = new Action("nextFrame") {
		public void accept(Visitor v) throws JASVisitorException { v.visitNEXT_FRAME(this); }
	};
	public static final Action NOT = new Action("not")  {
		public void accept(Visitor v) throws JASVisitorException { v.visitNOT(this); }
	};
	public static final Action OR = new Action("or") {
		public void accept(Visitor v) throws JASVisitorException { v.visitOR(this); }
	};
	public static final Action PLAY = new Action("play") {
		public void accept(Visitor v) throws JASVisitorException { v.visitPLAY(this); }
	};
	public static final Action POP = new Action("pop") {
		public void accept(Visitor v) throws JASVisitorException { v.visitPOP(this); }
	};
	public static final Action PREV_FRAME = new Action("prevFrame") {
		public void accept(Visitor v) throws JASVisitorException { v.visitPREV_FRAME(this); }
	};
	// ... see class PushAction ...
	public static final Action RANDOM_NUMBER = new Action("randomNumber") {
		public void accept(Visitor v) throws JASVisitorException { v.visitRANDOM_NUMBER(this); }
	};
	public static final Action REMOVE_SPRITE = new Action("removeSprite") {
		public void accept(Visitor v) throws JASVisitorException { v.visitREMOVE_SPRITE(this); }
	};
	public static final Action RETURN_VALUE = new Action("returnValue") {
		public void accept(Visitor v) throws JASVisitorException { v.visitRETURN_VALUE(this); }
	};
	public static final Action SET_MEMBER = new Action("setMember") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSET_MEMBER(this); }
	};
	public static final Action SET_PROPERTY = new Action("setProperty") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSET_PROPERTY(this); }
	};
	public static final Action SET_TARGET_2 = new Action("setTarget2") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSET_TARGET_2(this); }
	};
	// ... see class SetTargetAction ...
	public static final Action SET_VARIABLE = new Action("setVariable") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSET_VARIABLE(this); }
	};
	public static final Action SHIFT_LEFT = new Action("shiftLeft") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSHIFT_LEFT(this); }
	};
	public static final Action SHIFT_RIGHT = new Action("shiftRight") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSHIFT_RIGHT(this); }
	};
	public static final Action SHIFT_RIGHT_UNSIGNED = new Action("shiftRightUnsigned") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSHIFT_RIGHT_UNSIGNED(this); }
	};
	public static final Action START_DRAG = new Action("startDrag") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTART_DRAG(this); }
	};
	// ... see class StartFunction ...
	public static final Action START_WITH = new Action("startWith") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTART_WITH(this); }
	};
	public static final Action STOP = new Action("stop") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTOP(this); }
	};
	public static final Action STOP_SOUNDS = new Action("stopSounds") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTOP_SOUNDS(this); }
	};
	// ... see class StoreInRegisterAction ...
	public static final Action STRICT_EQUALS = new Action("strictEquals") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRICT_EQUALS(this); }
	};
	public static final Action STRING_EQUALS = new Action("stringEquals") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRING_EQUALS(this); }
	};
	public static final Action STRING_GREATER_THAN = new Action("stringGreaterThan") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRING_GREATER_THAN(this); }
	};
	public static final Action STRING_LENGTH = new Action("stringLength") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRING_LENGTH(this); }
	};
	public static final Action STRING_LENGTH_MB = new Action("stringLengthMB") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRING_LENGTH_MB(this); }
	};
	public static final Action STRING_LESS_THAN = new Action("stringLessThan") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSTRING_LESS_THAN(this); }
	};
	public static final Action SUBSTRING = new Action("substring") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSUBSTRING(this); }
	};
	public static final Action SUBSTRING_MB = new Action("substringMB") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSUBSTRING_MB(this); }
	};
	public static final Action SUBTRACT = new Action("subtract") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSUBTRACT(this); }
	};
	public static final Action SWAP = new Action("swap") {
		public void accept(Visitor v) throws JASVisitorException { v.visitSWAP(this); }
	};
	public static final Action TOGGLE_QUALITY = new Action("toggleQuality") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTOGGLE_QUALITY(this); }
	};
	public static final Action TO_INTEGER = new Action("toInteger") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTO_INTEGER(this); }
	};
	public static final Action TRACE = new Action("trace") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTRACE(this); }
	};
	public static final Action TYPED_ADD = new Action("typedAdd") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTYPED_ADD(this); }
	};
	public static final Action TYPED_EQUALS = new Action("typedEquals") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTYPED_EQUALS(this); }
	};
	public static final Action TYPED_LESS_THAN = new Action("typedLessThan") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTYPED_LESS_THAN(this); }
	};
	public static final Action TYPE_OF = new Action("typeOf") {
		public void accept(Visitor v) throws JASVisitorException { v.visitTYPE_OF(this); }
	};
	// ... see class UnknownAction ...
	// ... see class WaitForFrameAction ...
	// ... see class WaitForFrame2Action ...
}
