
package uk.co.badgersinfoil.jactionscript;

import java.util.Set;
import java.util.Iterator;

public class ActionHandle {
	private Action op;
	private ActionHandle prev;
	private ActionHandle next;
	private Set targeters = null; // lazily constructed to save space

	public ActionHandle() {
	}

	public ActionHandle(Action op) {
		this.op = op;
	}

	public void setAction(Action op) {
		this.op = op;
	}
	public Action getAction() {
		return op;
	}

	public void setNext(ActionHandle next) {
		this.next = next;
	}
	public ActionHandle getNext() {
		return next;
	}
	public void setPrev(ActionHandle prev) {
		this.prev = prev;
	}
	public ActionHandle getPrev() {
		return prev;
	}

	/**
	 * Do any BranchAction objects reference this handle?
	 */
	public boolean hasTargeters() {
		return targeters!=null && !targeters.isEmpty();
	}

	/**
	 * Return an iterator over any BranchAction objects targeting this
	 * handle
	 */
	public Iterator targeters() {
		// TODO: empty iterator if targeters==null
		return targeters.iterator();
	}

	/**
	 * It's the responsibility of the {@link BranchAction} to keep the handles
	 * it targets up to date by calling this method.
	 */
	public void addTargeter(BranchAction targeter) {
		if (targeters == null) {
			targeters = new java.util.HashSet();
		}
		targeters.add(targeter);
	}

	/**
	 * Called by {@link BranchAction} to keep its targets up to date.
	 */
	public void removeTargeter(BranchAction targeter) {
		// TODO: throw exception if it wasn't there?
		targeters.remove(targeter);
	}

	public void accept(Visitor v) throws JASVisitorException {
		v.visitHandle(this);
	}

	/**
	 * Breaks references held by this objects to others to promote
	 * garbage-collection and to expose code incorrectly using stale
	 * handles.  {@link ActionList#remove(ActionHandle)} calls this method
	 * on its argument as the final step of removal.
	 */
	public void dispose() {
		op = null;
		targeters = null;
		prev = null;
		next = null;
	}
}
