
package uk.co.badgersinfoil.jactionscript;

/**
 * Abstract superclass for Actions that can cause the flow of control though
 * the ActionList to branch.
 */
public abstract class BranchAction extends Action {
	private ActionHandle target;

	public BranchAction(String mnemonic, ActionHandle target) {
		super(mnemonic);
		this.target = target;
		target.addTargeter(this);
	}

	public ActionHandle getTarget() {
		return target;
	}

	/**
	 * Sets the target of this branch to the given handle.
	 * This branch is not removed from the list of targeters of the old
	 * target ActionHandle; the calling code must handle this.
	 */
	public void setTarget(ActionHandle to) {
		target = to;
		to.addTargeter(this);
		
	}

	// TODO: maybe define dispose() in the superclass and call it
	//       whenever an Action is removed from from its ActionList.  (this
	//       method isn't actually being called from anywhere at the mo)

	/**
	 * Removes this branch from the list of targeters maintained by
	 * the target ActionHandle and then breaks the reference to the target.
	 */
	public void dispose() {
		target.removeTargeter(this);
		target = null;
	}
}
