
package uk.co.badgersinfoil.jactionscript;


/**
 * @see ActionList#accept(Visitor)
 */
public interface Visitor {

	// TODO: maybe visitHandle() should be in a different inerface also
	// implemented by SWFActionsVisitor?

	/**
	 * the handle of a Action is always visited before the Action itself.
	 * (The implementation of visitHandle() doesn't have to visit the
	 * Action referenced by the handle, this is done elsewhere.)
	 */
	public void visitHandle(ActionHandle handle) throws JASVisitorException;

	/**
	 * Called after the last Action in a list is visited
	 */
	public void visitEnd(ActionList list) throws JASVisitorException;

	public void visitADD(Action op) throws JASVisitorException;
	public void visitAND(Action op) throws JASVisitorException;
	public void visitASCII_TO_CHAR(Action op) throws JASVisitorException;
	public void visitASCII_TO_CHAR_MB(Action op) throws JASVisitorException;
	public void visitBIT_AND(Action op) throws JASVisitorException;
	public void visitBIT_OR(Action op) throws JASVisitorException;
	public void visitBIT_XOR(Action op) throws JASVisitorException;
	public void visitCALL(Action op) throws JASVisitorException;
	public void visitCALL_FUNCTION(Action op) throws JASVisitorException;
	public void visitCALL_METHOD(Action op) throws JASVisitorException;
	public void visitCHAR_MB_TO_ASCII(Action op) throws JASVisitorException;
	public void visitCHAR_TO_ASCII(Action op) throws JASVisitorException;
	public void visitCLONE_SPRITE(Action op) throws JASVisitorException;
	public void visitCONCAT(Action op) throws JASVisitorException;
	public void visitCONVERT_TO_NUMBER(Action op) throws JASVisitorException;
	public void visitCONVERT_TO_STRING(Action op) throws JASVisitorException;
	public void visitDECREMENT(Action op) throws JASVisitorException;
	public void visitDEFINE_LOCAL(Action op) throws JASVisitorException;
	public void visitDEFINE_LOCAL_VALUE(Action op) throws JASVisitorException;
	public void visitDELETE_PROPERTY(Action op) throws JASVisitorException;
	public void visitDELETE_THREAD_VARS(Action op) throws JASVisitorException;
	public void visitDIVIDE(Action op) throws JASVisitorException;
	public void visitDUPLICATE(Action op) throws JASVisitorException;
	public void visitEND_DRAG(Action op) throws JASVisitorException;
	public void visitENUMERATE(Action op) throws JASVisitorException;
	public void visitENUMERATE_OBJECT(Action op) throws JASVisitorException;
	public void visitEQUALS(Action op) throws JASVisitorException;
	public void visitGET_MEMBER(Action op) throws JASVisitorException;
	public void visitGET_PROPERTY(Action op) throws JASVisitorException;
	public void visitGET_TARGET_PATH(Action op) throws JASVisitorException;
	public void visitGET_TIME(Action op) throws JASVisitorException;
	public void visitGET_URL2(GetURL2Action op) throws JASVisitorException;
	public void visitGET_URL(GetURLAction op) throws JASVisitorException;
	public void visitGET_VARIABLE(Action op) throws JASVisitorException;
	public void visitGOTO_FRAME2(GotoFrame2Action op) throws JASVisitorException;
	public void visitGOTO_FRAME(GotoFrameAction op) throws JASVisitorException;
	public void visitGOTO_LABEL(GotoLabelAction op) throws JASVisitorException;
	public void visitGREATER_THAN(Action op) throws JASVisitorException;
	public void visitIF_JUMP(IfJumpAction op) throws JASVisitorException;
	public void visitINCREMENT(Action op) throws JASVisitorException;
	public void visitINIT_ARRAY(Action op) throws JASVisitorException;
	public void visitINIT_OBJECT(Action op) throws JASVisitorException;
	public void visitINSTANCE_OF(Action op) throws JASVisitorException;
	public void visitJUMP(JumpAction op) throws JASVisitorException;
	public void visitLESS_THAN(Action op) throws JASVisitorException;
	public void visitLOOKUP(LookupAction op) throws JASVisitorException;
	public void visitLOOKUP_TABLE(LookupTableAction op) throws JASVisitorException;
	public void visitMODULO(Action op) throws JASVisitorException;
	public void visitMULTIPLY(Action op) throws JASVisitorException;
	public void visitNEW_METHOD(Action op) throws JASVisitorException;
	public void visitNEW_OBJECT(Action op) throws JASVisitorException;
	public void visitNEXT_FRAME(Action op) throws JASVisitorException;
	public void visitNOT(Action op) throws JASVisitorException;
	public void visitOR(Action op) throws JASVisitorException;
	public void visitPLAY(Action op) throws JASVisitorException;
	public void visitPOP(Action op) throws JASVisitorException;
	public void visitPREV_FRAME(Action op) throws JASVisitorException;
	public void visitPUSH(PushAction op) throws JASVisitorException;
	public void visitRANDOM_NUMBER(Action op) throws JASVisitorException;
	public void visitREMOVE_SPRITE(Action op) throws JASVisitorException;
	public void visitRETURN_VALUE(Action op) throws JASVisitorException;
	public void visitSET_MEMBER(Action op) throws JASVisitorException;
	public void visitSET_PROPERTY(Action op) throws JASVisitorException;
	public void visitSET_TARGET_2(Action op) throws JASVisitorException;
	public void visitSET_TARGET(SetTargetAction op) throws JASVisitorException;
	public void visitSET_VARIABLE(Action op) throws JASVisitorException;
	public void visitSHIFT_LEFT(Action op) throws JASVisitorException;
	public void visitSHIFT_RIGHT(Action op) throws JASVisitorException;
	public void visitSHIFT_RIGHT_UNSIGNED(Action op) throws JASVisitorException;
	public void visitSTART_DRAG(Action op) throws JASVisitorException;
	public void visitSTART_FUNCTION(StartFunctionAction op) throws JASVisitorException;
	public void visitSTART_WITH(Action op) throws JASVisitorException;
	public void visitSTOP(Action op) throws JASVisitorException;
	public void visitSTOP_SOUNDS(Action op) throws JASVisitorException;
	public void visitSTORE_IN_REGISTER(StoreInRegisterAction op) throws JASVisitorException;
	public void visitSTRICT_EQUALS(Action op) throws JASVisitorException;
	public void visitSTRING_EQUALS(Action op) throws JASVisitorException;
	public void visitSTRING_GREATER_THAN(Action op) throws JASVisitorException;
	public void visitSTRING_LENGTH(Action op) throws JASVisitorException;
	public void visitSTRING_LENGTH_MB(Action op) throws JASVisitorException;
	public void visitSTRING_LESS_THAN(Action op) throws JASVisitorException;
	public void visitSUBSTRING(Action op) throws JASVisitorException;
	public void visitSUBSTRING_MB(Action op) throws JASVisitorException;
	public void visitSUBTRACT(Action op) throws JASVisitorException;
	public void visitSWAP(Action op) throws JASVisitorException;
	public void visitTOGGLE_QUALITY(Action op) throws JASVisitorException;
	public void visitTO_INTEGER(Action op) throws JASVisitorException;
	public void visitTRACE(Action op) throws JASVisitorException;
	public void visitTYPED_ADD(Action op) throws JASVisitorException;
	public void visitTYPED_EQUALS(Action op) throws JASVisitorException;
	public void visitTYPED_LESS_THAN(Action op) throws JASVisitorException;
	public void visitTYPE_OF(Action op) throws JASVisitorException;
	public void visitUNKNOWN(UnknownAction op) throws JASVisitorException;
	public void visitWAIT_FOR_FRAME(WaitForFrameAction op) throws JASVisitorException;
	public void visitWAIT_FOR_FRAME_2(WaitForFrame2Action op) throws JASVisitorException;
}
