/*
 * DebugStackListener.java
 *
 * Copyright (c) 2004 David Holroyd
 *
 * $Revision: 1.2 $
 */


package uk.co.badgersinfoil.jactionscript.analysis;

import uk.co.badgersinfoil.jactionscript.*;
import java.util.List;
import java.util.Iterator;
import java.io.PrintStream;


// TODO: should be called LoggingStackListener

/**
 * Prints stuff about some of the actions encountered -- implementation not
 * complete or consistent.
 */
public class DebugStackListener extends FilterStackListener {

	PrintStream out = System.out;

	public DebugStackListener() {
		super(NullStackListener.VALUE);
	}

	public DebugStackListener(StackListener listener) {
		super(listener);
	}

	public void visitHandle(ActionHandle handle) throws JASVisitorException {
		super.visitHandle(handle);
	}

	public void visitEnd(ActionList list) throws JASVisitorException {
		super.visitEnd(list);
	}

	private void out(Action action) {
		out.print(action.getMnemonic());
		out.print(" ");
	}
	private void out(Operand val) {
		if (val.knownType()) {
			if (val.knownValue()) {
				if (val.getType() == Operand.ASType.STRING) {
					out.print('"');
					out.print(val.getValue());
					out.print('"');
				} else {
					out.print(val.getValue());
				}
			} else {
				out.print(val.getType());
			}
		} else {
			out.print("[?]");
		}
	}
	private void out(String s) {
		out.print(s);
	}
	private void out(int i) {
		out.print(i);
	}
	private void out(Action action, Operand left, String symbol, Operand right, Operand result) {
		out(action);
		out(left);
		out(" ");
		out(symbol);
		out(" ");
		out(right);
		out(" => ");
		out(result);
		out.println();
	}
	private void out(Action action, Operand val, Operand result) {
		out(action);
		out(val);
		out(" => ");
		out(result);
		out.println();
	}
	private void out(List list) {
		if (list == null) {
			out("[null]");
			return;
		}
		Iterator i = list.iterator();
		while (i.hasNext()) {
			out((Operand)i.next());
			if (i.hasNext()) out(", ");
		}
	}

	private void println() {
		out.println();
	}

	public void visitADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "+", right, result);
		super.visitADD(action, left, right, result);
	}
	public void visitAND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "&&", right, result);
		super.visitAND(action, left, right, result);
	}
	public void visitASCII_TO_CHAR(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitASCII_TO_CHAR(action, val, result);
	}
	public void visitASCII_TO_CHAR_MB(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitASCII_TO_CHAR_MB(action, val, result);
	}
	public void visitBIT_AND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "&", right, result);
		super.visitBIT_AND(action, left, right, result);
	}
	public void visitBIT_OR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "|", right, result);
		super.visitBIT_OR(action, left, right, result);
	}
	public void visitBIT_XOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "^", right, result);
		super.visitBIT_XOR(action, left, right, result);
	}
//	public void visitCALL(Action action) throws JASVisitorException {
//		
//		super.visitCALL(action);
//	}
	public void visitCALL_FUNCTION(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		out(action);
		out(name);
		out("(");
		if (args != null) out(args);
		out(")");
		out(" => ");
		out(result);
		println();
		super.visitCALL_FUNCTION(action, name, args, result);
	}
	public void visitCALL_METHOD(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitCALL_METHOD(action, name, args, result);
	}
	public void visitCHAR_MB_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitCHAR_MB_TO_ASCII(action, val, result);
	}
	public void visitCHAR_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitCHAR_TO_ASCII(action, val, result);
	}
//	public void visitCLONE_SPRITE(Action action) throws JASVisitorException {
//		
//		super.visitCLONE_SPRITE(action);
//	}
	public void visitCONCAT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "+", right, result);
		super.visitCONCAT(action, left, right, result);
	}
	public void visitCONVERT_TO_NUMBER(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitCONVERT_TO_NUMBER(action, val, result);
	}
	public void visitCONVERT_TO_STRING(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitCONVERT_TO_STRING(action, val, result);
	}
	public void visitDECREMENT(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitDECREMENT(action, val, result);
	}
	public void visitDEFINE_LOCAL(Action action, Operand name) throws JASVisitorException {
		out(action);
		out(name);
		println();
		super.visitDEFINE_LOCAL(action, name);
	}
	public void visitDEFINE_LOCAL_VALUE(Action action, Operand name, Operand value) throws JASVisitorException {
		out(action);
		out(name);
		out(" := ");
		out(value);
		println();
		super.visitDEFINE_LOCAL_VALUE(action, name, value);
	}
	public void visitDELETE_PROPERTY(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitDELETE_PROPERTY(action, name, obj, result);
	}
	public void visitDELETE_THREAD_VARS(Action action, Operand name) throws JASVisitorException {
		out(action);
		println();
		super.visitDELETE_THREAD_VARS(action, name);
	}
	public void visitDIVIDE(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "/", right, result);
		super.visitDIVIDE(action, left, right, result);
	}
	public void visitDUPLICATE(Action action, Operand val, Operand result1, Operand result2) throws JASVisitorException {
		out(action);
		println();
		super.visitDUPLICATE(action, val, result1, result2);
	}
	public void visitEND_DRAG(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitEND_DRAG(action);
	}
	public void visitENUMERATE(Action action, Operand name, List slots) throws JASVisitorException {
		out(action);
		println();
		super.visitENUMERATE(action, name, slots);
	}
	public void visitENUMERATE_OBJECT(Action action, Operand obj, List slots) throws JASVisitorException {
		out(action);
		println();
		super.visitENUMERATE_OBJECT(action, obj, slots);
	}
	public void visitEQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "==", right, result);
		super.visitEQUALS(action, left, right, result);
	}
	public void visitGET_MEMBER(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException {
		out(action);
		out(obj);
		out(".");
		out(name);
		out(" => ");
		out(result);
		println();
		super.visitGET_MEMBER(action, name, obj, result);
	}
	public void visitGET_PROPERTY(Action action, Operand index, Operand target, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitGET_PROPERTY(action, index, target, result);
	}
//	public void visitGET_TARGET_PATH(Action action) throws JASVisitorException {
//		
//		super.visitGET_TARGET_PATH(action);
//	}
	public void visitGET_TIME(Action action, Operand time) throws JASVisitorException {
		out(action);
		println();
		super.visitGET_TIME(action, time);
	}
	public void visitGET_URL2(GetURL2Action action, Operand target, Operand url) throws JASVisitorException {
		out(action);
		println();
		super.visitGET_URL2(action, target, url);
	}
	public void visitGET_URL(GetURLAction action) throws JASVisitorException {
		out(action);
		println();
		super.visitGET_URL(action);
	}
	public void visitGET_VARIABLE(Action action, Operand name, Operand result) throws JASVisitorException {
		out(action);
		out(name);
		out(" => ");
		out(result);
		println();
		super.visitGET_VARIABLE(action, name, result);
	}
	public void visitGOTO_FRAME2(GotoFrame2Action action, Operand frame) throws JASVisitorException {
		out(action);
		println();
		super.visitGOTO_FRAME2(action, frame);
	}
	public void visitGOTO_FRAME(GotoFrameAction action) throws JASVisitorException {
		out(action);
		println();
		super.visitGOTO_FRAME(action);
	}
//	public void visitGOTO_LABEL(GotoLabelAction action) throws JASVisitorException {
//		
//		super.visitGOTO_LABEL(action);
//	}
	public void visitGREATER_THAN(Action action, Operand right, Operand left, Operand result) throws JASVisitorException {
		out(action, left, ">", right, result);
		super.visitGREATER_THAN(action, right, left, result);
	}
	public void visitIF_JUMP(IfJumpAction action, Operand condition) throws JASVisitorException {
		out(action);
		println();
		super.visitIF_JUMP(action, condition);
	}
	public void visitINCREMENT(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitINCREMENT(action, val, result);
	}
//	public void visitINIT_ARRAY(Action action) throws JASVisitorException {
//		
//		super.visitINIT_ARRAY(action);
//	}
//	public void visitINIT_OBJECT(Action action) throws JASVisitorException {
//		
//		super.visitINIT_OBJECT(action);
//	}
	public void visitINSTANCE_OF(Action action, Operand ctor, Operand obj, Operand result) throws JASVisitorException {
		out(action, obj, "instanceof", ctor, result);
		super.visitINSTANCE_OF(action, ctor, obj, result);
	}
	public void visitJUMP(JumpAction action) throws JASVisitorException {
		out(action);
		println();
		super.visitJUMP(action);
	}
	public void visitLESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "<", right, result);
		super.visitLESS_THAN(action, left, right, result);
	}
	public void visitLOOKUP(LookupAction action, Operand result) throws JASVisitorException {
		out(action);
		out(action.getIndex());
		out(" => ");
		out(result);
		println();
		super.visitLOOKUP(action, result);
	}
	public void visitLOOKUP_TABLE(LookupTableAction action) throws JASVisitorException {
		out(action);
		LookupTable table = action.getLookupTable();
		for (int i=0 ; i<table.size() ; i++) {
			println();
			out("  "+i+": "+table.lookup(i));
		}
		println();
		super.visitLOOKUP_TABLE(action);
	}
	public void visitMODULO(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "%", right, result);
		super.visitMODULO(action, left, right, result);
	}
	public void visitMULTIPLY(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "*", right, result);
		super.visitMULTIPLY(action, left, right, result);
	}
//	public void visitNEW_METHOD(Action action) throws JASVisitorException {
//		
//		super.visitNEW_METHOD(action);
//	}
	public void visitNEW_OBJECT(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitNEW_OBJECT(action, name, args, result);
	}
	public void visitNEXT_FRAME(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitNEXT_FRAME(action);
	}
	public void visitNOT(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitNOT(action, val, result);
	}
	public void visitOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "||", right, result);
		super.visitOR(action, left, right, result);
	}
	public void visitPLAY(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitPLAY(action);
	}
	public void visitPOP(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitPOP(action);
	}
	public void visitPREV_FRAME(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitPREV_FRAME(action);
	}
	public void visitPUSH(PushAction action, List list) throws JASVisitorException {
		out(action);
		out(list);
		println();
		super.visitPUSH(action, list);
	}
	public void visitRANDOM_NUMBER(Action action, Operand max, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitRANDOM_NUMBER(action, max, result);
	}
	public void visitREMOVE_SPRITE(Action action, Operand target) throws JASVisitorException {
		out(action);
		println();
		super.visitREMOVE_SPRITE(action, target);
	}
	public void visitRETURN_VALUE(Action action, Operand returnVal) throws JASVisitorException {
		out(action);
		println();
		super.visitRETURN_VALUE(action, returnVal);
	}
	public void visitSET_MEMBER(Action action, Operand value, Operand name, Operand object) throws JASVisitorException {
		out(action);
		out(object);
		out(".");
		out(name);
		out(" := ");
		out(value);
		println();
		super.visitSET_MEMBER(action, value, name, object);
	}
	public void visitSET_PROPERTY(Action action, Operand value, Operand index, Operand target) throws JASVisitorException {
		out(action);
		println();
		super.visitSET_PROPERTY(action, value, index, target);
	}
	public void visitSET_TARGET_2(Action action, Operand target) throws JASVisitorException {
		out(action);
		println();
		super.visitSET_TARGET_2(action, target);
	}
	public void visitSET_TARGET(SetTargetAction action) throws JASVisitorException {
		out(action);
		println();
		super.visitSET_TARGET(action);
	}
	public void visitSET_VARIABLE(Action action, Operand name, Operand value) throws JASVisitorException {
		out(action);
		out(name);
		out(" := ");
		out(value);
		println();
		super.visitSET_VARIABLE(action, name, value);
	}
	public void visitSHIFT_LEFT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		out(action, value, "<<", count, result);
		super.visitSHIFT_LEFT(action, count, value, result);
	}
	public void visitSHIFT_RIGHT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		out(action, value, ">>", count, result);
		super.visitSHIFT_RIGHT(action, count, value, result);
	}
	public void visitSHIFT_RIGHT_UNSIGNED(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		out(action, value, ">>>", count, result);
		super.visitSHIFT_RIGHT_UNSIGNED(action, count, value, result);
	}
	public void visitSTART_DRAG(Action action, Operand target, Operand lookCentre, DragBox constraintBox) throws JASVisitorException {
		out(action);
		println();
		super.visitSTART_DRAG(action, target, lookCentre, constraintBox);
	}
	public void visitSTART_FUNCTION(StartFunctionAction action, Operand fn) throws JASVisitorException {
		out(action);
		println();
		super.visitSTART_FUNCTION(action, fn);
	}
//	public void visitSTART_WITH(Action action) throws JASVisitorException {
//		
//		super.visitSTART_WITH(action);
//	}
	public void visitSTOP(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitSTOP(action);
	}
	public void visitSTOP_SOUNDS(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitSTOP_SOUNDS(action);
	}
	public void visitSTORE_IN_REGISTER(StoreInRegisterAction action, Operand val) throws JASVisitorException {
		out(action);
		println();
		super.visitSTORE_IN_REGISTER(action, val);
	}
	public void visitSTRICT_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "===", right, result);
		super.visitSTRICT_EQUALS(action, left, right, result);
	}
	public void visitSTRING_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "==", right, result);
		super.visitSTRING_EQUALS(action, left, right, result);
	}
	public void visitSTRING_GREATER_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, ">", right, result);
		super.visitSTRING_GREATER_THAN(action, left, right, result);
	}
	public void visitSTRING_LENGTH(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitSTRING_LENGTH(action, val, result);
	}
	public void visitSTRING_LENGTH_MB(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitSTRING_LENGTH_MB(action, val, result);
	}
	public void visitSTRING_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "<", right, result);
		super.visitSTRING_LESS_THAN(action, left, right, result);
	}
	public void visitSUBSTRING(Action action, Operand count, Operand index, Operand val, Operand result) throws JASVisitorException {
		out(action);
		println();
		super.visitSUBSTRING(action, count, index, val, result);
	}
//	public void visitSUBSTRING_MB(Action action) throws JASVisitorException {
//		
//		super.visitSUBSTRING_MB(action);
//	}
	public void visitSUBTRACT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "-", right, result);
		super.visitSUBTRACT(action, left, right, result);
	}
	public void visitSWAP(Action action, Operand first, Operand second) throws JASVisitorException {
		out(action);
		println();
		super.visitSWAP(action, first, second);
	}
	public void visitTOGGLE_QUALITY(Action action) throws JASVisitorException {
		out(action);
		println();
		super.visitTOGGLE_QUALITY(action);
	}
	public void visitTO_INTEGER(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitTO_INTEGER(action, val, result);
	}
	public void visitTRACE(Action action, Operand val) throws JASVisitorException {
		out(action);
		println();
		super.visitTRACE(action, val);
	}
	public void visitTYPED_ADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "+", right, result);
		super.visitTYPED_ADD(action, left, right, result);
	}
	public void visitTYPED_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "==", right, result);
		super.visitTYPED_EQUALS(action, left, right, result);
	}
	public void visitTYPED_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		out(action, left, "<", right, result);
		super.visitTYPED_LESS_THAN(action, left, right, result);
	}
	public void visitTYPE_OF(Action action, Operand val, Operand result) throws JASVisitorException {
		out(action, val, result);
		super.visitTYPE_OF(action, val, result);
	}
//	public void visitUNKNOWN(UnknownAction action) throws JASVisitorException {
//		
//		super.visitUNKNOWN(action);
//	}
//	public void visitWAIT_FOR_FRAME(WaitForFrameAction action) throws JASVisitorException {
//		
//		super.visitWAIT_FOR_FRAME(action);
//	}
//	public void visitWAIT_FOR_FRAME_2(WaitForFrame2Action action) throws JASVisitorException {
//		
//		super.visitWAIT_FOR_FRAME_2(action);
//	}
}
