
package uk.co.badgersinfoil.jactionscript.analysis;

import uk.co.badgersinfoil.jactionscript.*;
import java.util.List;

/**
 * Implements all methods of {@link StackListener} as pass-throughs.
 */
public class FilterStackListener implements StackListener {

	private StackListener listener;

	public FilterStackListener(StackListener listener) {
		this.listener = listener;
	}

	public void visitHandle(ActionHandle handle) throws JASVisitorException {
		listener.visitHandle(handle);
	}

	public void visitEnd(ActionList list) throws JASVisitorException {
		listener.visitEnd(list);
	}

	public void visitADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitADD(action, left, right, result);
	}
	public void visitAND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitAND(action, left, right, result);
	}
	public void visitASCII_TO_CHAR(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitASCII_TO_CHAR(action, val, result);
	}
	public void visitASCII_TO_CHAR_MB(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitASCII_TO_CHAR_MB(action, val, result);
	}
	public void visitBIT_AND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitBIT_AND(action, left, right, result);
	}
	public void visitBIT_OR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitBIT_OR(action, left, right, result);
	}
	public void visitBIT_XOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitBIT_XOR(action, left, right, result);
	}
//	public void visitCALL(Action action) throws JASVisitorException {
//		listener.visitCALL(action);
//	}
	public void visitCALL_FUNCTION(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		listener.visitCALL_FUNCTION(action, name, args, result);
	}
	public void visitCALL_METHOD(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		listener.visitCALL_METHOD(action, name, args, result);
	}
	public void visitCHAR_MB_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitCHAR_MB_TO_ASCII(action, val, result);
	}
	public void visitCHAR_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitCHAR_TO_ASCII(action, val, result);
	}
//	public void visitCLONE_SPRITE(Action action) throws JASVisitorException {
//		listener.visitCLONE_SPRITE(action);
//	}
	public void visitCONCAT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitCONCAT(action, left, right, result);
	}
	public void visitCONVERT_TO_NUMBER(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitCONVERT_TO_NUMBER(action, val, result);
	}
	public void visitCONVERT_TO_STRING(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitCONVERT_TO_STRING(action, val, result);
	}
	public void visitDECREMENT(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitDECREMENT(action, val, result);
	}
	public void visitDEFINE_LOCAL(Action action, Operand name) throws JASVisitorException {
		listener.visitDEFINE_LOCAL(action, name);
	}
	public void visitDEFINE_LOCAL_VALUE(Action action, Operand name, Operand value) throws JASVisitorException {
		listener.visitDEFINE_LOCAL_VALUE(action, name, value);
	}
	public void visitDELETE_PROPERTY(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException {
		listener.visitDELETE_PROPERTY(action, name, obj, result);
	}
	public void visitDELETE_THREAD_VARS(Action action, Operand name) throws JASVisitorException {
		listener.visitDELETE_THREAD_VARS(action, name);
	}
	public void visitDIVIDE(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitDIVIDE(action, left, right, result);
	}
	public void visitDUPLICATE(Action action, Operand val, Operand result1, Operand result2) throws JASVisitorException {
		listener.visitDUPLICATE(action, val, result1, result2);
	}
	public void visitEND_DRAG(Action action) throws JASVisitorException {
		listener.visitEND_DRAG(action);
	}
	public void visitENUMERATE(Action action, Operand name, List slots) throws JASVisitorException {
		listener.visitENUMERATE(action, name, slots);
	}
	public void visitENUMERATE_OBJECT(Action action, Operand obj, List slots) throws JASVisitorException {
		listener.visitENUMERATE_OBJECT(action, obj, slots);
	}
	public void visitEQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitEQUALS(action, left, right, result);
	}
	public void visitGET_MEMBER(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException {
		listener.visitGET_MEMBER(action, name, obj, result);
	}
	public void visitGET_PROPERTY(Action action, Operand index, Operand target, Operand result) throws JASVisitorException {
		listener.visitGET_PROPERTY(action, index, target, result);
	}
//	public void visitGET_TARGET_PATH(Action action) throws JASVisitorException {
//		listener.visitGET_TARGET_PATH(action);
//	}
	public void visitGET_TIME(Action action, Operand time) throws JASVisitorException {
		listener.visitGET_TIME(action, time);
	}
	public void visitGET_URL2(GetURL2Action action, Operand target, Operand url) throws JASVisitorException {
		listener.visitGET_URL2(action, target, url);
	}
	public void visitGET_URL(GetURLAction action) throws JASVisitorException {
		listener.visitGET_URL(action);
	}
	public void visitGET_VARIABLE(Action action, Operand name, Operand result) throws JASVisitorException {
		listener.visitGET_VARIABLE(action, name, result);
	}
	public void visitGOTO_FRAME2(GotoFrame2Action action, Operand frame) throws JASVisitorException {
		listener.visitGOTO_FRAME2(action, frame);
	}
	public void visitGOTO_FRAME(GotoFrameAction action) throws JASVisitorException {
		listener.visitGOTO_FRAME(action);
	}
//	public void visitGOTO_LABEL(GotoLabelAction action) throws JASVisitorException {
//		listener.visitGOTO_LABEL(action);
//	}
	public void visitGREATER_THAN(Action action, Operand right, Operand left, Operand result) throws JASVisitorException {
		listener.visitGREATER_THAN(action, right, left, result);
	}
	public void visitIF_JUMP(IfJumpAction action, Operand condition) throws JASVisitorException {
		listener.visitIF_JUMP(action, condition);
	}
	public void visitINCREMENT(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitINCREMENT(action, val, result);
	}
//	public void visitINIT_ARRAY(Action action) throws JASVisitorException {
//		listener.visitINIT_ARRAY(action);
//	}
//	public void visitINIT_OBJECT(Action action) throws JASVisitorException {
//		listener.visitINIT_OBJECT(action);
//	}
	public void visitINSTANCE_OF(Action action, Operand ctor, Operand obj, Operand result) throws JASVisitorException {
		listener.visitINSTANCE_OF(action, ctor, obj, result);
	}
	public void visitJUMP(JumpAction action) throws JASVisitorException {
		listener.visitJUMP(action);
	}
	public void visitLESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitLESS_THAN(action, left, right, result);
	}
	public void visitLOOKUP(LookupAction action, Operand result) throws JASVisitorException {
		listener.visitLOOKUP(action, result);
	}
	public void visitLOOKUP_TABLE(LookupTableAction action) throws JASVisitorException {
		listener.visitLOOKUP_TABLE(action);
	}
	public void visitMODULO(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitMODULO(action, left, right, result);
	}
	public void visitMULTIPLY(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitMULTIPLY(action, left, right, result);
	}
//	public void visitNEW_METHOD(Action action) throws JASVisitorException {
//		listener.visitNEW_METHOD(action);
//	}
	public void visitNEW_OBJECT(Action action, Operand name, List args, Operand result) throws JASVisitorException {
		listener.visitNEW_OBJECT(action, name, args, result);
	}
	public void visitNEXT_FRAME(Action action) throws JASVisitorException {
		listener.visitNEXT_FRAME(action);
	}
	public void visitNOT(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitNOT(action, val, result);
	}
	public void visitOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitOR(action, left, right, result);
	}
	public void visitPLAY(Action action) throws JASVisitorException {
		listener.visitPLAY(action);
	}
	public void visitPOP(Action action) throws JASVisitorException {
		listener.visitPOP(action);
	}
	public void visitPREV_FRAME(Action action) throws JASVisitorException {
		listener.visitPREV_FRAME(action);
	}
	public void visitPUSH(PushAction action, List list) throws JASVisitorException {
		listener.visitPUSH(action, list);
	}
	public void visitRANDOM_NUMBER(Action action, Operand max, Operand result) throws JASVisitorException {
		listener.visitRANDOM_NUMBER(action, max, result);
	}
	public void visitREMOVE_SPRITE(Action action, Operand target) throws JASVisitorException {
		listener.visitREMOVE_SPRITE(action, target);
	}
	public void visitRETURN_VALUE(Action action, Operand returnVal) throws JASVisitorException {
		listener.visitRETURN_VALUE(action, returnVal);
	}
	public void visitSET_MEMBER(Action action, Operand value, Operand name, Operand object) throws JASVisitorException {
		listener.visitSET_MEMBER(action, value, name, object);
	}
	public void visitSET_PROPERTY(Action action, Operand value, Operand index, Operand target) throws JASVisitorException {
		listener.visitSET_PROPERTY(action, value, index, target);
	}
	public void visitSET_TARGET_2(Action action, Operand target) throws JASVisitorException {
		listener.visitSET_TARGET_2(action, target);
	}
	public void visitSET_TARGET(SetTargetAction action) throws JASVisitorException {
		listener.visitSET_TARGET(action);
	}
	public void visitSET_VARIABLE(Action action, Operand name, Operand value) throws JASVisitorException {
		listener.visitSET_VARIABLE(action, name, value);
	}
	public void visitSHIFT_LEFT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		listener.visitSHIFT_LEFT(action, count, value, result);
	}
	public void visitSHIFT_RIGHT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		listener.visitSHIFT_RIGHT(action, count, value, result);
	}
	public void visitSHIFT_RIGHT_UNSIGNED(Action action, Operand count, Operand value, Operand result) throws JASVisitorException {
		listener.visitSHIFT_RIGHT_UNSIGNED(action, count, value, result);
	}
	public void visitSTART_DRAG(Action action, Operand target, Operand lookCentre, DragBox constraintBox) throws JASVisitorException {
		listener.visitSTART_DRAG(action, target, lookCentre, constraintBox);
	}
	public void visitSTART_FUNCTION(StartFunctionAction action, Operand fn) throws JASVisitorException {
		listener.visitSTART_FUNCTION(action, fn);
	}
//	public void visitSTART_WITH(Action action) throws JASVisitorException {
//		listener.visitSTART_WITH(action);
//	}
	public void visitSTOP(Action action) throws JASVisitorException {
		listener.visitSTOP(action);
	}
	public void visitSTOP_SOUNDS(Action action) throws JASVisitorException {
		listener.visitSTOP_SOUNDS(action);
	}
	public void visitSTORE_IN_REGISTER(StoreInRegisterAction action, Operand val) throws JASVisitorException {
		listener.visitSTORE_IN_REGISTER(action, val);
	}
	public void visitSTRICT_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitSTRICT_EQUALS(action, left, right, result);
	}
	public void visitSTRING_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitSTRING_EQUALS(action, left, right, result);
	}
	public void visitSTRING_GREATER_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitSTRING_GREATER_THAN(action, left, right, result);
	}
	public void visitSTRING_LENGTH(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitSTRING_LENGTH(action, val, result);
	}
	public void visitSTRING_LENGTH_MB(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitSTRING_LENGTH_MB(action, val, result);
	}
	public void visitSTRING_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitSTRING_LESS_THAN(action, left, right, result);
	}
	public void visitSUBSTRING(Action action, Operand count, Operand index, Operand val, Operand result) throws JASVisitorException {
		listener.visitSUBSTRING(action, count, index, val, result);
	}
//	public void visitSUBSTRING_MB(Action action) throws JASVisitorException {
//		listener.visitSUBSTRING_MB(action);
//	}
	public void visitSUBTRACT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitSUBTRACT(action, left, right, result);
	}
	public void visitSWAP(Action action, Operand first, Operand second) throws JASVisitorException {
		listener.visitSWAP(action, first, second);
	}
	public void visitTOGGLE_QUALITY(Action action) throws JASVisitorException {
		listener.visitTOGGLE_QUALITY(action);
	}
	public void visitTO_INTEGER(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitTO_INTEGER(action, val, result);
	}
	public void visitTRACE(Action action, Operand val) throws JASVisitorException {
		listener.visitTRACE(action, val);
	}
	public void visitTYPED_ADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitTYPED_ADD(action, left, right, result);
	}
	public void visitTYPED_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitTYPED_EQUALS(action, left, right, result);
	}
	public void visitTYPED_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException {
		listener.visitTYPED_LESS_THAN(action, left, right, result);
	}
	public void visitTYPE_OF(Action action, Operand val, Operand result) throws JASVisitorException {
		listener.visitTYPE_OF(action, val, result);
	}
//	public void visitUNKNOWN(UnknownAction action) throws JASVisitorException {
//		listener.visitUNKNOWN(action);
//	}
//	public void visitWAIT_FOR_FRAME(WaitForFrameAction action) throws JASVisitorException {
//		listener.visitWAIT_FOR_FRAME(action);
//	}
//	public void visitWAIT_FOR_FRAME_2(WaitForFrame2Action action) throws JASVisitorException {
//		listener.visitWAIT_FOR_FRAME_2(action);
//	}
}
