
package uk.co.badgersinfoil.jactionscript.analysis;

import uk.co.badgersinfoil.jactionscript.*;
import java.util.List;

/**
 * A StackListener is informed when SimpleStackSimulator updates its simulated
 * stack.
 */
public interface StackListener {

	public void visitHandle(ActionHandle handle) throws JASVisitorException;

	public void visitEnd(ActionList list) throws JASVisitorException;

	public void visitADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitAND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitASCII_TO_CHAR(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitASCII_TO_CHAR_MB(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitBIT_AND(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitBIT_OR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitBIT_XOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
//	public void visitCALL(Action action) throws JASVisitorException;
	public void visitCALL_FUNCTION(Action action, Operand name, List args, Operand result) throws JASVisitorException;
	public void visitCALL_METHOD(Action action, Operand name, List args, Operand result) throws JASVisitorException;
	public void visitCHAR_MB_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitCHAR_TO_ASCII(Action action, Operand val, Operand result) throws JASVisitorException;
//	public void visitCLONE_SPRITE(Action action) throws JASVisitorException;
	public void visitCONCAT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitCONVERT_TO_NUMBER(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitCONVERT_TO_STRING(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitDECREMENT(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitDEFINE_LOCAL(Action action, Operand name) throws JASVisitorException;
	public void visitDEFINE_LOCAL_VALUE(Action action, Operand name, Operand value) throws JASVisitorException;
	public void visitDELETE_PROPERTY(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException;
	public void visitDELETE_THREAD_VARS(Action action, Operand name) throws JASVisitorException;
	public void visitDIVIDE(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitDUPLICATE(Action action, Operand val, Operand result1, Operand result2) throws JASVisitorException;
	public void visitEND_DRAG(Action action) throws JASVisitorException;
	public void visitENUMERATE(Action action, Operand name, List slots) throws JASVisitorException;
	public void visitENUMERATE_OBJECT(Action action, Operand obj, List slots) throws JASVisitorException;
	public void visitEQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitGET_MEMBER(Action action, Operand name, Operand obj, Operand result) throws JASVisitorException;
	public void visitGET_PROPERTY(Action action, Operand index, Operand target, Operand result) throws JASVisitorException;
//	public void visitGET_TARGET_PATH(Action action) throws JASVisitorException;
	public void visitGET_TIME(Action action, Operand time) throws JASVisitorException;
	public void visitGET_URL2(GetURL2Action action, Operand target, Operand url) throws JASVisitorException;
	public void visitGET_URL(GetURLAction action) throws JASVisitorException;
	public void visitGET_VARIABLE(Action action, Operand name, Operand result) throws JASVisitorException;
	public void visitGOTO_FRAME2(GotoFrame2Action action, Operand frame) throws JASVisitorException;
	public void visitGOTO_FRAME(GotoFrameAction action) throws JASVisitorException;
//	public void visitGOTO_LABEL(GotoLabelAction action) throws JASVisitorException;
	public void visitGREATER_THAN(Action action, Operand right, Operand left, Operand result) throws JASVisitorException;
	public void visitIF_JUMP(IfJumpAction action, Operand condition) throws JASVisitorException;
	public void visitINCREMENT(Action action, Operand val, Operand result) throws JASVisitorException;
//	public void visitINIT_ARRAY(Action action) throws JASVisitorException;
//	public void visitINIT_OBJECT(Action action) throws JASVisitorException;
	public void visitINSTANCE_OF(Action action, Operand ctor, Operand obj, Operand result) throws JASVisitorException;
	public void visitJUMP(JumpAction action) throws JASVisitorException;
	public void visitLESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitLOOKUP(LookupAction action, Operand result) throws JASVisitorException;
	public void visitLOOKUP_TABLE(LookupTableAction action) throws JASVisitorException;
	public void visitMODULO(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitMULTIPLY(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
//	public void visitNEW_METHOD(Action action) throws JASVisitorException;
	public void visitNEW_OBJECT(Action action, Operand name, List args, Operand result) throws JASVisitorException;
	public void visitNEXT_FRAME(Action action) throws JASVisitorException;
	public void visitNOT(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitOR(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitPLAY(Action action) throws JASVisitorException;
	public void visitPOP(Action action) throws JASVisitorException;
	public void visitPREV_FRAME(Action action) throws JASVisitorException;
	public void visitPUSH(PushAction action, List list) throws JASVisitorException;
	public void visitRANDOM_NUMBER(Action action, Operand max, Operand result) throws JASVisitorException;
	public void visitREMOVE_SPRITE(Action action, Operand target) throws JASVisitorException;
	public void visitRETURN_VALUE(Action action, Operand returnVal) throws JASVisitorException;
	public void visitSET_MEMBER(Action action, Operand value, Operand name, Operand object) throws JASVisitorException;
	public void visitSET_PROPERTY(Action action, Operand value, Operand index, Operand target) throws JASVisitorException;
	public void visitSET_TARGET_2(Action action, Operand target) throws JASVisitorException;
	public void visitSET_TARGET(SetTargetAction action) throws JASVisitorException;
	public void visitSET_VARIABLE(Action action, Operand name, Operand value) throws JASVisitorException;
	public void visitSHIFT_LEFT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException;
	public void visitSHIFT_RIGHT(Action action, Operand count, Operand value, Operand result) throws JASVisitorException;
	public void visitSHIFT_RIGHT_UNSIGNED(Action action, Operand count, Operand value, Operand result) throws JASVisitorException;
	public static class DragBox {
		public Operand coord1, coord2, coord3, coord4;
		public DragBox(Operand coord1, Operand coord2, Operand coord3, Operand coord4) {
			this.coord1 = coord1;  this.coord2 = coord2;  this.coord3 = coord3;  this.coord4 = coord4;
		}
	}
	/** @param constraintBox null, if unconstrained */
	public void visitSTART_DRAG(Action action, Operand target, Operand lookCentre, DragBox constraintBox) throws JASVisitorException;
	/** @param fn will only be non-null for anonymous functions */
	public void visitSTART_FUNCTION(StartFunctionAction action, Operand fn) throws JASVisitorException;
//	public void visitSTART_WITH(Action action) throws JASVisitorException;
	public void visitSTOP(Action action) throws JASVisitorException;
	public void visitSTOP_SOUNDS(Action action) throws JASVisitorException;
	public void visitSTORE_IN_REGISTER(StoreInRegisterAction action, Operand val) throws JASVisitorException;
	public void visitSTRICT_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitSTRING_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitSTRING_GREATER_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitSTRING_LENGTH(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitSTRING_LENGTH_MB(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitSTRING_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitSUBSTRING(Action action, Operand count, Operand index, Operand val, Operand result) throws JASVisitorException;
//	public void visitSUBSTRING_MB(Action action) throws JASVisitorException;
	public void visitSUBTRACT(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitSWAP(Action action, Operand first, Operand second) throws JASVisitorException;
	public void visitTOGGLE_QUALITY(Action action) throws JASVisitorException;
	public void visitTO_INTEGER(Action action, Operand val, Operand result) throws JASVisitorException;
	public void visitTRACE(Action action, Operand val) throws JASVisitorException;
	public void visitTYPED_ADD(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitTYPED_EQUALS(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitTYPED_LESS_THAN(Action action, Operand left, Operand right, Operand result) throws JASVisitorException;
	public void visitTYPE_OF(Action action, Operand val, Operand result) throws JASVisitorException;
//	public void visitUNKNOWN(UnknownAction action) throws JASVisitorException;
//	public void visitWAIT_FOR_FRAME(WaitForFrameAction action) throws JASVisitorException;
//	public void visitWAIT_FOR_FRAME_2(WaitForFrame2Action action) throws JASVisitorException;
}
