
package uk.co.badgersinfoil.jactionscript.javaswf;

import com.anotherbigidea.flash.interfaces.SWFActions;

import uk.co.badgersinfoil.jactionscript.*;

import java.util.Map;
import java.util.List;
import java.io.IOException;

public class BytecodeSWFActions implements SWFActions {

	private ActionList list;
	private PushAction currentPush;
	private Map jumpTargets = new java.util.HashMap();
	private List contextStack = new java.util.ArrayList();

	public BytecodeSWFActions() {
		reset();
	}

	/**
	 * Returns the list that's been gathering bytecodes from each call
	 * to a SWFActions method.
	 */
	public ActionList getList() {
		return list;
	}

	private void append(Action op) {
		list.append(op);
		currentPush = null;
	}

	private void enterContext(ActionList next) {
		contextStack.add(list);
		list = next;
	}

	private void reset() {
		contextStack.clear();
		list = new ActionList();
		currentPush = null;
	}

	private void leaveContext() {
		list = (ActionList)contextStack.remove(contextStack.size()-1);
	}

	private ActionHandle getBranchTarget(String jumpLabel) {
		ActionHandle target = (ActionHandle)jumpTargets.get(jumpLabel);
		if (target == null) {
			target = new ActionHandle();
			jumpTargets.put(jumpLabel, target);
		}
		return target;
	}

	private void setBranchTarget(String jumpLabel) {
		ActionHandle target = (ActionHandle)jumpTargets.get(jumpLabel);
		if (target == null) {
			target = list.peekNext();
			jumpTargets.put(jumpLabel, target);
		} else {
			list.pokeNext(target);
		}
	}

	private void doPush(Object val) {
		PushAction op = currentPush;
		if (op == null) {
			op = new PushAction(val);
			append(op);
		} else {
			op.add(val);
		}
		currentPush = op;
	}

	public void add() throws IOException {
		append(ActionConstants.ADD);
	}
	public void and() throws IOException {
		append(ActionConstants.AND);
	}
	public void asciiToChar() throws IOException {
		append(ActionConstants.ASCII_TO_CHAR);
	}
	public void asciiToCharMB() throws IOException {
		append(ActionConstants.ASCII_TO_CHAR_MB);
	}
	public void bitAnd() throws IOException {
		append(ActionConstants.BIT_AND);
	}
	public void bitOr() throws IOException {
		append(ActionConstants.BIT_OR);
	}
	public void bitXor() throws IOException {
		append(ActionConstants.BIT_XOR);
	}
	public void blob(byte[] blob) throws IOException {
		throw new RuntimeException("Didn't expect blob() to be called");
//		append(ActionConstants.);
	}
	public void call() throws IOException {
		append(ActionConstants.CALL);
	}
	public void callFunction() throws IOException {
		append(ActionConstants.CALL_FUNCTION);
	}
	public void callMethod() throws IOException {
		append(ActionConstants.CALL_METHOD);
	}
	public void charMBToAscii() throws IOException {
		append(ActionConstants.CHAR_MB_TO_ASCII);
	}
	public void charToAscii() throws IOException {
		append(ActionConstants.CHAR_TO_ASCII);
	}
	public void cloneSprite() throws IOException {
		append(ActionConstants.CLONE_SPRITE);
	}
	public void comment(String comment) throws IOException {
		// ignore comment
		//   (could make an attribute of subsequent ActionHandle, but
		//   that would lose comments at the end of the bytecode)
	}
	public void concat() throws IOException {
		append(ActionConstants.CONCAT);
	}
	public void convertToNumber() throws IOException {
		append(ActionConstants.CONVERT_TO_NUMBER);
	}
	public void convertToString() throws IOException {
		append(ActionConstants.CONVERT_TO_STRING);
	}
	public void decrement() throws IOException {
		append(ActionConstants.DECREMENT);
	}
	public void defineLocal() throws IOException {
		append(ActionConstants.DEFINE_LOCAL);
	}
	public void defineLocalValue() throws IOException {
		append(ActionConstants.DEFINE_LOCAL_VALUE);
	}
	public void deleteProperty() throws IOException {
		append(ActionConstants.DELETE_PROPERTY);
	}
	public void deleteThreadVars() throws IOException {
		append(ActionConstants.DELETE_THREAD_VARS);
	}
	public void divide() throws IOException {
		append(ActionConstants.DIVIDE);
	}
	public void done() throws IOException {
		// TODO: "End of all action blocks"
	}
	public void duplicate() throws IOException {
		append(ActionConstants.DUPLICATE);
	}
	public void end() throws IOException {
		// TODO: "End of actions"
	}
	public void endBlock() throws IOException {
		leaveContext();
		// TODO ?
	}
	public void endDrag() throws IOException {
		append(ActionConstants.END_DRAG);
	}
	public void enumerate() throws IOException {
		append(ActionConstants.ENUMERATE);
	}
	public void enumerateObject() throws IOException {
		append(ActionConstants.ENUMERATE_OBJECT);
	}
	public void equals() throws IOException {
		append(ActionConstants.EQUALS);
	}
	public void getMember() throws IOException {
		append(ActionConstants.GET_MEMBER);
	}
	public void getProperty() throws IOException {
		append(ActionConstants.GET_PROPERTY);
	}
	public void getTargetPath() throws IOException {
		append(ActionConstants.GET_TARGET_PATH);
	}
	public void getTime() throws IOException {
		append(ActionConstants.GET_TIME);
	}
	public void getURL(int sendVars, int loadMode) throws IOException {
		append(new GetURL2Action(sendVars, loadMode));
	}
	public void getURL(String url, String target) throws IOException {
		append(new GetURLAction(url, target));
	}
	public void getVariable() throws IOException {
		append(ActionConstants.GET_VARIABLE);
	}
	public void gotoFrame(boolean play) throws IOException {
		append(new GotoFrame2Action(play));
	}
	public void gotoFrame(int frame) throws IOException {
		append(new GotoFrameAction(frame));
	}
	public void gotoFrame(String label) throws IOException {
		append(new GotoLabelAction(label));
	}
	public void greaterThan() throws IOException {
		append(ActionConstants.GREATER_THAN);
	}
	public void ifJump(String jumpLabel) throws IOException {
		append(new IfJumpAction(getBranchTarget(jumpLabel)));
	}
	public void increment() throws IOException {
		append(ActionConstants.INCREMENT);
	}
	public void initArray() throws IOException {
		append(ActionConstants.INIT_ARRAY);
	}
	public void initObject() throws IOException {
		append(ActionConstants.INIT_OBJECT);
	}
	public void instanceOf() throws IOException {
		append(ActionConstants.INSTANCE_OF);
	}
	public void jump(String jumpLabel) throws IOException {
		append(new JumpAction(getBranchTarget(jumpLabel)));
	}
	public void jumpLabel(String jumpLabel) throws IOException {
		// not an Action -- javaswf has worked out the next Action will be
		// the target of a branch
		setBranchTarget(jumpLabel);
	}
	public void lessThan() throws IOException {
		append(ActionConstants.LESS_THAN);
	}
	public void lookup(int dictionaryIndex) throws IOException {
		append(new LookupAction(dictionaryIndex));
	}
	public void lookupTable(String[] values) throws IOException {
		append(new LookupTableAction(new LookupTable(values)));
	}
	public void modulo() throws IOException {
		append(ActionConstants.MODULO);
	}
	public void multiply() throws IOException {
		append(ActionConstants.MULTIPLY);
	}
	public void newMethod() throws IOException {
		append(ActionConstants.NEW_METHOD);
	}
	public void newObject() throws IOException {
		append(ActionConstants.NEW_OBJECT);
	}
	public void nextFrame() throws IOException {
		append(ActionConstants.NEXT_FRAME);
	}
	public void not() throws IOException {
		append(ActionConstants.NOT);
	}
	public void or() throws IOException {
		append(ActionConstants.OR);
	}
	public void play() throws IOException {
		append(ActionConstants.PLAY);
	}
	public void pop() throws IOException {
		append(ActionConstants.POP);
	}
	public void prevFrame() throws IOException {
		append(ActionConstants.PREV_FRAME);
	}
	public void push(boolean value) throws IOException {
		doPush(new Boolean(value));
	}
	public void push(double value) throws IOException {
		doPush(new Double(value));
	}
	public void push(float value) throws IOException {
		doPush(new Float(value));
	}
	public void push(int value) throws IOException {
		doPush(new Integer(value));
	}
	public void push(String value) throws IOException {
		doPush(value);
	}
	public void pushNull() throws IOException {
		doPush(null);
	}
	public void pushRegister(int registerNumber) throws IOException {
		doPush(new Register(registerNumber));
	}
	public void pushUndefined() throws IOException {
		doPush(PushAction.UNDEFINED);
	}
	public void randomNumber() throws IOException {
		append(ActionConstants.RANDOM_NUMBER);
	}
	public void removeSprite() throws IOException {
		append(ActionConstants.REMOVE_SPRITE);
	}
	public void returnValue() throws IOException {
		append(ActionConstants.RETURN_VALUE);
	}
	public void setMember() throws IOException {
		append(ActionConstants.SET_MEMBER);
	}
	public void setProperty() throws IOException {
		append(ActionConstants.SET_PROPERTY);
	}
	public void setTarget() throws IOException {
		append(ActionConstants.SET_TARGET_2);
	}
	public void setTarget(String target) throws IOException {
		append(new SetTargetAction(target));
	}
	public void setVariable() throws IOException {
		append(ActionConstants.SET_VARIABLE);
	}
	public void shiftLeft() throws IOException {
		append(ActionConstants.SHIFT_LEFT);
	}
	public void shiftRight() throws IOException {
		append(ActionConstants.SHIFT_RIGHT);
	}
	public void shiftRightUnsigned() throws IOException {
		append(ActionConstants.SHIFT_RIGHT_UNSIGNED);
	}
	public void start(int flags) throws IOException {
		// TODO: ...?
		reset();
	}
	public void start(int flags, int keycode) throws IOException {
		// TODO: ...?
		reset();
	}
	public void startDrag() throws IOException {
		append(ActionConstants.START_DRAG);
	}
	public void startFunction(String name, String[] paramNames) throws IOException {
		ActionList next = new ActionList();
		append(new StartFunctionAction(name, paramNames, next));
		enterContext(next);
	}
	public void startWith() throws IOException {
		// TODO: needs special handling of block, per startFunction
		append(ActionConstants.START_WITH);
	}
	public void stop() throws IOException {
		append(ActionConstants.STOP);
	}
	public void stopSounds() throws IOException {
		append(ActionConstants.STOP_SOUNDS);
	}
	public void storeInRegister(int registerNumber) throws IOException {
		append(new StoreInRegisterAction(new Register(registerNumber)));
	}
	public void strictEquals() throws IOException {
		append(ActionConstants.STRICT_EQUALS);
	}
	public void stringEquals() throws IOException {
		append(ActionConstants.STRING_EQUALS);
	}
	public void stringGreaterThan() throws IOException {
		append(ActionConstants.STRING_GREATER_THAN);
	}
	public void stringLength() throws IOException {
		append(ActionConstants.STRING_LENGTH);
	}
	public void stringLengthMB() throws IOException {
		append(ActionConstants.STRING_LENGTH_MB);
	}
	public void stringLessThan() throws IOException {
		append(ActionConstants.STRING_LESS_THAN);
	}
	public void substring() throws IOException {
		append(ActionConstants.SUBSTRING);
	}
	public void substringMB() throws IOException {
		append(ActionConstants.SUBSTRING_MB);
	}
	public void subtract() throws IOException {
		append(ActionConstants.SUBTRACT);
	}
	public void swap() throws IOException {
		append(ActionConstants.SWAP);
	}
	public void toggleQuality() throws IOException {
		append(ActionConstants.TOGGLE_QUALITY);
	}
	public void toInteger() throws IOException {
		append(ActionConstants.TO_INTEGER);
	}
	public void trace() throws IOException {
		append(ActionConstants.TRACE);
	}
	public void typedAdd() throws IOException {
		append(ActionConstants.TYPED_ADD);
	}
	public void typedEquals() throws IOException {
		append(ActionConstants.TYPED_EQUALS);
	}
	public void typedLessThan() throws IOException {
		append(ActionConstants.TYPED_LESS_THAN);
	}
	public void typeOf() throws IOException {
		append(ActionConstants.TYPE_OF);
	}
	public void unknown(int code, byte[] data) throws IOException {
		append(new UnknownAction(code, data));
	}
	public void waitForFrame(int frameNumber, String jumpLabel) throws IOException {
		append(new WaitForFrameAction(frameNumber,
		                          getBranchTarget(jumpLabel)));
	}
	public void waitForFrame(String jumpLabel) throws IOException {
		append(new WaitForFrame2Action(getBranchTarget(jumpLabel)));
	}
}
