

package uk.co.badgersinfoil.jactionscript.javaswf;

import com.anotherbigidea.flash.interfaces.SWFActions;

import uk.co.badgersinfoil.jactionscript.*;

import java.util.Map;
import java.util.Iterator;
import java.io.IOException;

// TODO: how/where does start(int)/start(int,int) get called?

public class SWFActionsVisitor implements Visitor {
	private SWFActions actions;
	private Map jumpLabels = new java.util.HashMap();
	private int labelSequence = 0;

	public SWFActionsVisitor(SWFActions actions) {
		this.actions = actions;
	}

	private String getJumpLabelForTarget(ActionHandle target) {
		// pluggable naming strategy?  anyone care?
		String label = (String)jumpLabels.get(target);
		if (label == null) {
			label = "label" + (labelSequence++);
			jumpLabels.put(target, label);
		}
		return label;
	}

	private void handle(IOException e) throws JASVisitorException {
		throw new JASVisitorException(e);
	}

	public void visitHandle(ActionHandle handle) throws JASVisitorException {
		if (handle.hasTargeters()) {
			// stuff wants to jump to the Action referenced by this
			// handle, so javaswf wants a label before the Action,
			try {
				actions.jumpLabel(getJumpLabelForTarget(handle));
			} catch (IOException e) {
				handle(e);
			}
		}
	}

	public void visitEnd(ActionList list) throws JASVisitorException {
		try {
			actions.endBlock();
		} catch (IOException e) {
			handle(e);
		}
	}

	public void visitADD(Action op) throws JASVisitorException {
		try {
			actions.add();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitAND(Action op) throws JASVisitorException {
		try {
			actions.and();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitASCII_TO_CHAR(Action op) throws JASVisitorException {
		try {
			actions.asciiToChar();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitASCII_TO_CHAR_MB(Action op) throws JASVisitorException {
		try {
			actions.asciiToCharMB();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitBIT_AND(Action op) throws JASVisitorException {
		try {
			actions.bitAnd();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitBIT_OR(Action op) throws JASVisitorException {
		try {
			actions.bitOr();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitBIT_XOR(Action op) throws JASVisitorException {
		try {
			actions.bitXor();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCALL(Action op) throws JASVisitorException {
		try {
			actions.call();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCALL_FUNCTION(Action op) throws JASVisitorException {
		try {
			actions.callFunction();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCALL_METHOD(Action op) throws JASVisitorException {
		try {
			actions.callMethod();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCHAR_MB_TO_ASCII(Action op) throws JASVisitorException {
		try {
			actions.charMBToAscii();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCHAR_TO_ASCII(Action op) throws JASVisitorException {
		try {
			actions.charToAscii();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCLONE_SPRITE(Action op) throws JASVisitorException {
		try {
			actions.cloneSprite();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCONCAT(Action op) throws JASVisitorException {
		try {
			actions.concat();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCONVERT_TO_NUMBER(Action op) throws JASVisitorException {
		try {
			actions.convertToNumber();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitCONVERT_TO_STRING(Action op) throws JASVisitorException {
		try {
			actions.convertToString();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDECREMENT(Action op) throws JASVisitorException {
		try {
			actions.decrement();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDEFINE_LOCAL(Action op) throws JASVisitorException {
		try {
			actions.defineLocal();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDEFINE_LOCAL_VALUE(Action op) throws JASVisitorException {
		try {
			actions.defineLocalValue();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDELETE_PROPERTY(Action op) throws JASVisitorException {
		try {
			actions.deleteProperty();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDELETE_THREAD_VARS(Action op) throws JASVisitorException {
		try {
			actions.deleteThreadVars();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDIVIDE(Action op) throws JASVisitorException {
		try {
			actions.divide();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitDUPLICATE(Action op) throws JASVisitorException {
		try {
			actions.duplicate();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitEND_DRAG(Action op) throws JASVisitorException {
		try {
			actions.endDrag();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitENUMERATE(Action op) throws JASVisitorException {
		try {
			actions.enumerate();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitENUMERATE_OBJECT(Action op) throws JASVisitorException {
		try {
			actions.enumerateObject();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitEQUALS(Action op) throws JASVisitorException {
		try {
			actions.equals();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_MEMBER(Action op) throws JASVisitorException {
		try {
			actions.getMember();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_PROPERTY(Action op) throws JASVisitorException {
		try {
			actions.getProperty();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_TARGET_PATH(Action op) throws JASVisitorException {
		try {
			actions.getTargetPath();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_TIME(Action op) throws JASVisitorException {
		try {
			actions.getTime();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_URL2(GetURL2Action op) throws JASVisitorException {
		try {
			actions.getURL(op.getSendVars(), op.getLoadMode());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_URL(GetURLAction op) throws JASVisitorException {
		try {
			actions.getURL(op.getURL(), op.getTarget());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGET_VARIABLE(Action op) throws JASVisitorException {
		try {
			actions.getVariable();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGOTO_FRAME2(GotoFrame2Action op) throws JASVisitorException {
		try {
			actions.gotoFrame(op.getPlay());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGOTO_FRAME(GotoFrameAction op) throws JASVisitorException {
		try {
			actions.gotoFrame(op.getFrame());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGOTO_LABEL(GotoLabelAction op) throws JASVisitorException {
		try {
			actions.gotoFrame(op.getLabel());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitGREATER_THAN(Action op) throws JASVisitorException {
		try {
			actions.greaterThan();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitIF_JUMP(IfJumpAction op) throws JASVisitorException {
		try {
			actions.ifJump(getJumpLabelForTarget(op.getTarget()));
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitINCREMENT(Action op) throws JASVisitorException {
		try {
			actions.increment();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitINIT_ARRAY(Action op) throws JASVisitorException {
		try {
			actions.initArray();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitINIT_OBJECT(Action op) throws JASVisitorException {
		try {
			actions.initObject();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitINSTANCE_OF(Action op) throws JASVisitorException {
		try {
			actions.instanceOf();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitJUMP(JumpAction op) throws JASVisitorException {
		try {
			actions.jump(getJumpLabelForTarget(op.getTarget()));
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitLESS_THAN(Action op) throws JASVisitorException {
		try {
			actions.lessThan();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitLOOKUP(LookupAction op) throws JASVisitorException {
		try {
			actions.lookup(op.getIndex());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitLOOKUP_TABLE(LookupTableAction op) throws JASVisitorException {
		try {
			actions.lookupTable(op.getLookupTable().toArray());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitMODULO(Action op) throws JASVisitorException {
		try {
			actions.modulo();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitMULTIPLY(Action op) throws JASVisitorException {
		try {
			actions.multiply();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitNEW_METHOD(Action op) throws JASVisitorException {
		try {
			actions.newMethod();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitNEW_OBJECT(Action op) throws JASVisitorException {
		try {
			actions.newObject();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitNEXT_FRAME(Action op) throws JASVisitorException {
		try {
			actions.nextFrame();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitNOT(Action op) throws JASVisitorException {
		try {
			actions.not();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitOR(Action op) throws JASVisitorException {
		try {
			actions.or();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitPLAY(Action op) throws JASVisitorException {
		try {
			actions.play();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitPOP(Action op) throws JASVisitorException {
		try {
			actions.pop();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitPREV_FRAME(Action op) throws JASVisitorException {
		try {
			actions.prevFrame();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitPUSH(PushAction op) throws JASVisitorException {
		Iterator i = op.valueIterator();
		try {
			while (i.hasNext()) {
				Object val = i.next();
				if (val == null) {
					actions.pushNull();
				} else if (val == PushAction.UNDEFINED) {
					actions.pushUndefined();
				} else if (val instanceof Boolean) {
					actions.push(((Boolean)val).booleanValue());
				} else if (val instanceof Double) {
					actions.push(((Double)val).doubleValue());
				} else if (val instanceof Float) {
					actions.push(((Float)val).floatValue());
				} else if (val instanceof Integer) {
					actions.push(((Integer)val).intValue());
				} else if (val instanceof String) {
					actions.push((String)val);
				} else if (val instanceof Register) {
					actions.pushRegister(((Register)val).number());
				}
			}
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitRANDOM_NUMBER(Action op) throws JASVisitorException {
		try {
			actions.randomNumber();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitREMOVE_SPRITE(Action op) throws JASVisitorException {
		try {
			actions.removeSprite();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitRETURN_VALUE(Action op) throws JASVisitorException {
		try {
			actions.returnValue();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSET_MEMBER(Action op) throws JASVisitorException {
		try {
			actions.setMember();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSET_PROPERTY(Action op) throws JASVisitorException {
		try {
			actions.setProperty();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSET_TARGET_2(Action op) throws JASVisitorException {
		try {
			actions.setTarget();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSET_TARGET(SetTargetAction op) throws JASVisitorException {
		try {
			actions.setTarget(op.getTarget());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSET_VARIABLE(Action op) throws JASVisitorException {
		try {
			actions.setVariable();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSHIFT_LEFT(Action op) throws JASVisitorException {
		try {
			actions.shiftLeft();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSHIFT_RIGHT(Action op) throws JASVisitorException {
		try {
			actions.shiftRight();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSHIFT_RIGHT_UNSIGNED(Action op) throws JASVisitorException {
		try {
			actions.shiftRightUnsigned();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTART_DRAG(Action op) throws JASVisitorException {
		try {
			actions.startDrag();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTART_FUNCTION(StartFunctionAction op) throws JASVisitorException {
		try {
			actions.startFunction(op.getName(), op.getParamNames());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTART_WITH(Action op) throws JASVisitorException {
		try {
			actions.startWith();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTOP(Action op) throws JASVisitorException {
		try {
			actions.stop();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTOP_SOUNDS(Action op) throws JASVisitorException {
		try {
			actions.stopSounds();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTORE_IN_REGISTER(StoreInRegisterAction op) throws JASVisitorException {
		try {
			actions.storeInRegister(op.getRegister().number());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRICT_EQUALS(Action op) throws JASVisitorException {
		try {
			actions.strictEquals();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRING_EQUALS(Action op) throws JASVisitorException {
		try {
			actions.stringEquals();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRING_GREATER_THAN(Action op) throws JASVisitorException {
		try {
			actions.stringGreaterThan();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRING_LENGTH(Action op) throws JASVisitorException {
		try {
			actions.stringLength();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRING_LENGTH_MB(Action op) throws JASVisitorException {
		try {
			actions.stringLengthMB();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSTRING_LESS_THAN(Action op) throws JASVisitorException {
		try {
			actions.stringLessThan();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSUBSTRING(Action op) throws JASVisitorException {
		try {
			actions.substring();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSUBSTRING_MB(Action op) throws JASVisitorException {
		try {
			actions.substringMB();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSUBTRACT(Action op) throws JASVisitorException {
		try {
			actions.subtract();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitSWAP(Action op) throws JASVisitorException {
		try {
			actions.swap();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTOGGLE_QUALITY(Action op) throws JASVisitorException {
		try {
			actions.toggleQuality();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTO_INTEGER(Action op) throws JASVisitorException {
		try {
			actions.toInteger();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTRACE(Action op) throws JASVisitorException {
		try {
			actions.trace();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTYPED_ADD(Action op) throws JASVisitorException {
		try {
			actions.typedAdd();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTYPED_EQUALS(Action op) throws JASVisitorException {
		try {
			actions.typedEquals();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTYPED_LESS_THAN(Action op) throws JASVisitorException {
		try {
			actions.typedLessThan();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitTYPE_OF(Action op) throws JASVisitorException {
		try {
			actions.typeOf();
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitUNKNOWN(UnknownAction op) throws JASVisitorException {
		try {
			actions.unknown(op.getCode(), op.getData());
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitWAIT_FOR_FRAME(WaitForFrameAction op) throws JASVisitorException {
		try {
			actions.waitForFrame(op.getFrameNumber(),
		                     getJumpLabelForTarget(op.getTarget()));
		} catch (IOException e) {
			handle(e);
		}
	}
	public void visitWAIT_FOR_FRAME_2(WaitForFrame2Action op) throws JASVisitorException {
		try {
			actions.waitForFrame(getJumpLabelForTarget(op.getTarget()));
		} catch (IOException e) {
			handle(e);
		}
	}
}
